# -*- coding: utf-8 -*-
import re
import execjs
from loguru import logger
from aiohttp import ClientSession
from typing import Optional,NoReturn
from execjs._external_runtime import ExternalRuntime

JS_CODE = """
function a(r, o) {
    for (var t = 0; t < o.length - 2; t += 3) {
        var a = o.charAt(t + 2);
        a = a >= "a" ? a.charCodeAt(0) - 87 : Number(a),
        a = "+" === o.charAt(t + 1) ? r >>> a: r << a,
        r = "+" === o.charAt(t) ? r + a & 4294967295 : r ^ a
    }
    return r
}
var C = null;
var token = function(r, _gtk) {
    var o = r.length;
    o > 30 && (r = "" + r.substr(0, 10) + r.substr(Math.floor(o / 2) - 5, 10) + r.substring(r.length, r.length - 10));
    var t = void 0,
    t = null !== C ? C: (C = _gtk || "") || "";
    for (var e = t.split("."), h = Number(e[0]) || 0, i = Number(e[1]) || 0, d = [], f = 0, g = 0; g < r.length; g++) {
        var m = r.charCodeAt(g);
        128 > m ? d[f++] = m: (2048 > m ? d[f++] = m >> 6 | 192 : (55296 === (64512 & m) && g + 1 < r.length && 56320 === (64512 & r.charCodeAt(g + 1)) ? (m = 65536 + ((1023 & m) << 10) + (1023 & r.charCodeAt(++g)), d[f++] = m >> 18 | 240, d[f++] = m >> 12 & 63 | 128) : d[f++] = m >> 12 | 224, d[f++] = m >> 6 & 63 | 128), d[f++] = 63 & m | 128)
    }
    for (var S = h,
    u = "+-a^+6",
    l = "+-3^+b+-f",
    s = 0; s < d.length; s++) S += d[s],
    S = a(S, u);

    return S = a(S, l),
    S ^= i,
    0 > S && (S = (2147483647 & S) + 2147483648),
    S %= 1e6,
    S.toString() + "." + (S ^ h)
}
"""

class Trans:
    
    __session: Optional[ClientSession] = None
    __headers: dict = None
    __token: Optional[str] = None
    __gtk: Optional[str] = None
    __javascript: ExternalRuntime.Context = execjs.compile(JS_CODE)
    
    def __init__(
            self,
            session: ClientSession = None,
            headers: dict = None,
        ):
        if headers is None:
            headers = {
                'User-Agent':
                'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/71.0.3578.98 Safari/537.36'
            }
        self.__session=session
        self.__headers=headers

    async def initialization(
            self,
            session: ClientSession = None,
        ) -> NoReturn:
        self.__session=session
        try:
            # 获得token和gtk
            # 必须要加载两次保证token是最新的，否则会出现998的错误
            await self.loadMainPage()
            await self.loadMainPage()
        except Exception as e:
            raise e
    

    async def loadMainPage(self) -> NoReturn:
        url = 'https://fanyi.baidu.com'
        async with self.__session.get(url, headers=self.__headers) as r:
            text = await r.text()
            self.__token = re.findall(r"token: '(.*?)',", text)[0]
            self.__gtk = re.findall(r"window.gtk = '(.*?)';", text)[0]


    async def langdetect(self, query):
        url = 'https://fanyi.baidu.com/langdetect'
        data = {'query': query}
        try:
            async with self.__session.post(url=url, data=data) as r:
                json = await r.json()
                if 'msg' in json and json['msg'] == 'success':
                    return json['lan']
                return None
        except Exception as e:
            raise e
        

    async def dictionary(self, query, dst='zh', src=None) -> Optional[str]:
        url = 'https://fanyi.baidu.com/v2transapi'
        sign = self.__javascript.call('token', query, self.__gtk)
        if not src:
            src = await self.langdetect(query)
        data = {
            'from': src,
            'to': dst,
            'query': query,
            'simple_means_flag': 3,
            'sign': sign,
            'token': self.__token,
        }
        try:
            async with self.__session.post(url=url, data=data) as r:
                if r.status == 200:
                    json = await r.json()
                    if 'error' in json:
                        raise Exception('baidu sdk error: {}'.format(json['error']))
                    return json['trans_result']['data'][0]['dst']
                return None
        except Exception as e:
            raise e
        


import asyncio
from loguru import logger
from types import TracebackType
from typing import Optional,Type
from aiohttp import ClientSession

from choc.ddp.translate import Trans

def is_contains_chinese(strs):
    for _char in strs:
        if '\u4e00' <= _char <= '\u9fa5':
            return True
    return False

class Choco:
    
    trans : Optional[Trans] = None
    session: Optional[ClientSession] = None
    level_map: dict = {
        1: ["zh", "en", "de", "zh"],
        2: ["zh", "en", "de", "jp" ,"pt", "zh"],
        3: ["zh", "en", "de", "jp" ,"pt", "it", "pl", "bul", "est", "zh"]
    }
    
    def __init__(self,trans: Optional[Trans] = None) -> None:
        if trans is None:
            trans = Trans(session=self.session)
        self.trans = trans
    
    async def __aenter__(self) -> "Choco":
        self.session = ClientSession()
        await self.trans.initialization(session = self.session)
        return self

    async def __aexit__(
            self, 
            exc_type: Optional[Type[BaseException]],
            exc_val: Optional[BaseException],
            exc_tb: Optional[TracebackType]
        ) -> None:
        await self.session.close()
        
        
    async def transfer(self,content: str,level: int) -> str:
        target = self.level_map[level]
        for i in range(len(target)):
            if i == len(target)-1:
                break;
            content = await self.trans.dictionary(content,dst=target[i+1],src=target[i])
        return content
    
    async def deduplication(
            self,
            content: str = None,
            level: int = 1
        ) -> str:
        """_summary_

        Args:
            content (str, optional): _description_. Defaults to None.
            level (int, optional): _description_. Defaults to 1.
                1: zh -> en -> de -> zh
                2: zh -> en -> de -> jp -> pt -> zh
                3: zh -> en -> de -> jp -> pt -> it -> pl -> bul -> est -> zh
        """
        if content is None or not is_contains_chinese(content):
            logger.error(f"内容无效，仅支持中文")
        if level not in range(1,4):
            logger.error(f"level: {level}超出范围")
        return await self.transfer(content,level)
        
    
    
async def test():
    async with Choco() as c:
       await c.deduplication(input("待去重：\n"),3)


if __name__ == '__main__':
    loop = asyncio.get_event_loop()
    loop.run_until_complete(test())
    


# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['choc', 'choc.ddp']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.8.1,<4.0.0', 'loguru>=0.6.0,<0.7.0', 'pyexecjs>=1.5.1,<2.0.0']

setup_kwargs = {
    'name': 'choc-ddp',
    'version': '0.1.2',
    'description': 'chocolate deduplication',
    'long_description': '# choc_ddp\n\n```python\npip install choc-ddp\n\n#推荐\npoetry add choc-ddp\n```\n\n示例\n```python\nimport asyncio \nfrom loguru import logger\nfrom choc.ddp.chocolate import Choco\n\nasync def test():\n    async with Choco() as c:\n       resp = await c.deduplication(input("待去重：\\n"),3)\n       logger.info(resp)\n       \nif __name__ == \'__main__\':\n    loop = asyncio.get_event_loop()\n    loop.run_until_complete(test())\n```',
    'author': 'AnjoyLi',
    'author_email': '1589631311@qq.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
